@@ -35,11 +35,11 @@ Net::Jabber::Bot - Automated Bot creation with safeties
 
 =head1 VERSION
 
-Version 2.0.2
+Version 2.0.3
 
 =cut
 
-our $VERSION = '2.0.2';
+our $VERSION = '2.0.3';
 
 =head1 SYNOPSIS
 
@@ -54,7 +54,7 @@ All someone should have to know and define in the program away from the object i
 
 =item 1. Config - Where to connect, how often to do things, timers, etc
 
-=item 2. A subroutine to be called by the bot object when a new message comes in. 
+=item 2. A subroutine to be called by the bot object when a new message comes in.
 
 =item 3. A subroutine to be called by the bot object every so often that lets the user do background activities (check logs, monitor web pages, etc.),
 
@@ -68,7 +68,7 @@ The object at present has the following enforced safeties as long as you do not
 
 =item 2. Limits messages per second, configurable at start up, (Max is 5 per second) by requiring a sleep timer in the message sending subroutine each time one is sent.
 
-=item 3. Endless loops of responding to self prevented by now allowing the bot message processing subroutine to know about messages from self 
+=item 3. Endless loops of responding to self prevented by now allowing the bot message processing subroutine to know about messages from self
 
 =item 4. Forum join grace period to prevent bot from reacting to historical messages
 
@@ -123,7 +123,7 @@ The following initialization variables can be passed. Only marked variables are
 
     safety_mode = (1,0,'on','off')
 
-Determines if the bot safety features are turned on and enforced. This mode is on by default. Many of the safety features are here to assure you do not crash your favorite jabber server with floods, etc. DO NOT turn it off unless you're sure you know what you're doing (not just Sledge Hammer ceratin) 
+Determines if the bot safety features are turned on and enforced. This mode is on by default. Many of the safety features are here to assure you do not crash your favorite jabber server with floods, etc. DO NOT turn it off unless you're sure you know what you're doing (not just Sledge Hammer ceratin)
 
 =item B<server>
 
@@ -186,7 +186,7 @@ Boolean value as to whether we should ignore messages sent to us from the jabber
 
 =item B<ignore_self_messages>
 
-Boolean value as to whether we should ignore messages sent by us. 
+Boolean value as to whether we should ignore messages sent by us.
 
 BE CAREFUL if you turn this on!!! Turning this on risks potentially endless loops. If you're going to do this, please be sure safety is turned on at least initially.
 
@@ -224,14 +224,16 @@ sub BUILD {
 
     $forum_join_grace{$obj_ID} = 20;
 
+    $jabber_client{$obj_ID} = $arg_ref->{'jabber_client'};
+
     # Safety mode is on unless they feed us 0 or off explicitly
     $safety_mode{$obj_ID} = $arg_ref->{'safety_mode'};
-    if(!defined $safety_mode{$obj_ID} 
+    if(!defined $safety_mode{$obj_ID}
        || $safety_mode{$obj_ID} !~ m/^\s*off\s*$/i
        || $safety_mode{$obj_ID} != 0) {
-	$safety_mode{$obj_ID} = 0;
+    $safety_mode{$obj_ID} = 0;
     } else {
-	$safety_mode{$obj_ID} = 1;
+    $safety_mode{$obj_ID} = 1;
     }
 
     $connection_hash{$obj_ID}{'server'} = $arg_ref->{'server'};
@@ -249,13 +251,13 @@ sub BUILD {
     $bot_background_activity{$obj_ID} = $arg_ref->{'background_activity'};
 
     $loop_sleep_time{$obj_ID} = $arg_ref->{'loop_sleep_time'}
-	or $loop_sleep_time{$obj_ID} = 5;
+    or $loop_sleep_time{$obj_ID} = 5;
 
     $process_timeout{$obj_ID} = $arg_ref->{'process_timeout'}
-	or $process_timeout{$obj_ID} = 5;
+    or $process_timeout{$obj_ID} = 5;
 
     $connection_hash{$obj_ID}{'from_full'} =
-	"$connection_hash{$obj_ID}{'username'}\@$connection_hash{$obj_ID}{'server'}/$connection_hash{$obj_ID}{'alias'}";
+    "$connection_hash{$obj_ID}{'username'}\@$connection_hash{$obj_ID}{'server'}/$connection_hash{$obj_ID}{'alias'}";
 
     $ignore_messages{$obj_ID}{ignore_server_messages} = $arg_ref->{'ignore_server_messages'};
     $ignore_messages{$obj_ID}{ignore_server_messages} = 1 if(!defined $ignore_messages{$obj_ID}{ignore_server_messages});
@@ -266,24 +268,24 @@ sub BUILD {
     $forums_and_responses{$obj_ID} = $arg_ref->{'forums_and_responses'};
 
     my $out_messages_per_second = $arg_ref->{'out_messages_per_second'};
-    $out_messages_per_second = 5 
-	if(!defined $out_messages_per_second || $out_messages_per_second <= 0); # Can't be < 0 or undef
+    $out_messages_per_second = 5
+    if(!defined $out_messages_per_second || $out_messages_per_second <= 0); # Can't be < 0 or undef
 
     $message_delay{$obj_ID} = 1 / $out_messages_per_second;
 
     # Set the maximum chunk size to fed value if it's reasonable.
     if(defined $arg_ref->{'max_message_size'} && $arg_ref->{'max_message_size'} > 100) { # Can't be < 100 (don't be silly)
-	$max_message_size{$obj_ID} = $arg_ref->{'max_message_size'};
+    $max_message_size{$obj_ID} = $arg_ref->{'max_message_size'};
     } else {
-	$max_message_size{$obj_ID} = 1,000,000; # Set it to one meg if not specified.
+    $max_message_size{$obj_ID} = 1,000,000; # Set it to one meg if not specified.
     }
 
     # Set the maximum messages per day limit to fed value if it's within reason
     if(defined $arg_ref->{'max_messages_per_hour'} && $arg_ref->{'max_messages_per_hour'} > 0) { # Must be undef and > 0
-	$max_messages_per_hour{$obj_ID} = $arg_ref->{'max_messages_per_hour'};
+    $max_messages_per_hour{$obj_ID} = $arg_ref->{'max_messages_per_hour'};
     } else {
-	# Set it to a really big number (Safety will catch if you're not dumb enough to disable it.)
-	$max_messages_per_hour{$obj_ID} = 1,000,000;
+    # Set it to a really big number (Safety will catch if you're not dumb enough to disable it.)
+    $max_messages_per_hour{$obj_ID} = 1,000,000;
     }
 
     # Initialize today's message count.
@@ -300,7 +302,7 @@ sub BUILD {
     $max_message_size{$obj_ID} = 1000 if($max_message_size{$obj_ID} > 1000);
 
     # More than 4,000 messages a day is a little excessive.
-    $max_messages_per_hour{$obj_ID} = 125 if($max_message_size{$obj_ID} > 166); 
+    $max_messages_per_hour{$obj_ID} = 125 if($max_message_size{$obj_ID} > 166);
 
     # Should not be responding to self messages to prevent loops.
     $ignore_messages{$obj_ID}{ignore_self_messages} = 1;
@@ -311,24 +313,24 @@ sub BUILD {
 
 Sets up the special message handling and then initializes the connection.
 
-=cut 
+=cut
 
 sub START {
     my ($self, $obj_ID, $arg_ref) = @_;
-    CreateJabberNamespaces();
-    $self->InitJabber();
+#    CreateJabberNamespaces(); # Trying to get it to handle version messages.
+   $self->InitJabber();
 }
 
-# Sets up special name space handling. 
+# Sets up special name space handling.
 sub CreateJabberNamespaces : PRIVATE {
     Net::Jabber::Namespaces::add_ns(  ns => "jabber:iq:version",
-				      tag   => "query",
-				      xpath => {
-						name       => { type=>'scalar' },
-						version  => { type=>'scalar' },
-						os      => { type=>'scalar' },
-					       }
-				   );
+                      tag   => "query",
+                      xpath => {
+                        name       => { type=>'scalar' },
+                        version  => { type=>'scalar' },
+                        os      => { type=>'scalar' },
+                           }
+                   );
 }
 
 # Return a code reference that will pass self in addition to arguements passed to callback code ref.
@@ -340,15 +342,19 @@ sub callback_maker : PRIVATE {
     return sub {return $Function->($self, @_);};
 }
 
-
 # Creates client object and manages connection. Called on new but also called by re-connect
 sub InitJabber : PRIVATE {
     my $self = shift;
     my $obj_ID = $self->_get_obj_id() or return;
 
-    DEBUG("new client object.");
-    my $connection = new Net::Jabber::Client();
-    $jabber_client{$obj_ID} = $connection;
+    # Determine if the object already exists and if not, create it.
+    my $client_exists = 1;
+    DEBUG("new client object."); 
+    if(!defined $jabber_client{$obj_ID}) { # If client was fed in for test?
+        $jabber_client{$obj_ID} = new Net::Jabber::Client();
+        $client_exists = 0;
+    }
+    my $connection = $jabber_client{$obj_ID};
 
     DEBUG("Set the call backs.");
 
@@ -362,8 +368,8 @@ sub InitJabber : PRIVATE {
     my $status = $connection->Connect(hostname=>$connection_hash{$obj_ID}{'server'} , port=>$connection_hash{$obj_ID}{'port'});
 
     if(!defined $status) {
-	ERROR("ERROR:  Jabber server is down or connection was not allowed: $!");
-	return;
+       ERROR("ERROR:  Jabber server is down or connection was not allowed: $!");
+       return;
     }
 
     DEBUG("Logging in... as user $connection_hash{$obj_ID}{'username'}/$connection_hash{$obj_ID}{'alias'}");
@@ -372,11 +378,11 @@ sub InitJabber : PRIVATE {
                                             resource=>$connection_hash{$obj_ID}{'alias'});
 
     if(!defined $auth_result[0] || $auth_result[0] ne "ok") {
-	ERROR("ERROR: Authorization failed:");
-	foreach my $result (@auth_result) {
-	    ERROR("$result");
-	}
-	return;
+    ERROR("ERROR: Authorization failed:");
+    foreach my $result (@auth_result) {
+        ERROR("$result");
+    }
+        return;
     }
 
     $connection_session_id{$obj_ID} = $connection->{SESSION}->{id};
@@ -399,8 +405,8 @@ sub InitJabber : PRIVATE {
 
 =item B<JoinForum>
 
-Joins a jabber forum and sleeps safety time. Also prevents the object 
-from responding to messages for a grace period in efforts to get it to 
+Joins a jabber forum and sleeps safety time. Also prevents the object
+from responding to messages for a grace period in efforts to get it to
 not respond to historical messages. This has failed sometimes.
 
 NOTE: No error detection for join failure is present at the moment. (TODO)
@@ -426,9 +432,9 @@ sub JoinForum {
 
 =item B<Process>
 
-Mostly calls it's client connection's "Process" call. 
+Mostly calls it's client connection's "Process" call.
 Also assures a timeout is enforced if not fed to the subroutine
-You really should not have to call this very often. 
+You really should not have to call this very often.
 You should mostly be calling Start() and just let the Bot kernel handle all this.
 
 =cut
@@ -439,7 +445,7 @@ sub Process { # Call connection process.
     my $timeout_seconds = shift;
 
     #If not passed explicitly
-    $timeout_seconds = $process_timeout{$obj_ID} if(!defined $timeout_seconds); 
+    $timeout_seconds = $process_timeout{$obj_ID} if(!defined $timeout_seconds);
 
     my $process_return = $jabber_client{$obj_ID}->Process($timeout_seconds);
     return $process_return;
@@ -468,28 +474,28 @@ sub Start {
     my $counter = 0; # Keep track of how many times we've looped. Not sure if we'll use this long term.
 
     while(1) { # Loop for ever!
-	# Process and re-connect if you have to.
-	my $reconnect_timeout = 1;
-	while(!defined $self->Process($process_timeout)) {
-	    Time::HiRes::sleep $reconnect_timeout++; # Timeout Progressiveley longer.
-	    my $message = "Disconnected from $connection_hash{$obj_ID}{'server'}:$connection_hash{$obj_ID}{'port'}"
-		. " as $connection_hash{$obj_ID}{'username'}."; 
-	    ERROR("$message Reconnecting...");
-	    $self->ReconnectToServer();
-	    next; # do not allow background to run till you've re-connected.
-	}
+    # Process and re-connect if you have to.
+    my $reconnect_timeout = 1;
+    while(!defined $self->Process($process_timeout)) {
+        Time::HiRes::sleep $reconnect_timeout++; # Timeout Progressiveley longer.
+        my $message = "Disconnected from $connection_hash{$obj_ID}{'server'}:$connection_hash{$obj_ID}{'port'}"
+        . " as $connection_hash{$obj_ID}{'username'}.";
+        ERROR("$message Reconnecting...");
+        $self->ReconnectToServer();
+        next; # do not allow background to run till you've re-connected.
+    }
         # Call background function
-	if(defined $background_subroutine && $last_background + $time_between_background_routines < time) {
-	    &$background_subroutine($self, ++$counter);
-	    $last_background = time;
-	}
+    if(defined $background_subroutine && $last_background + $time_between_background_routines < time) {
+        &$background_subroutine($self, ++$counter);
+        $last_background = time;
+    }
         Time::HiRes::sleep $message_delay;
     }
 }
 
 =item B<ReconnectToServer>
 
-You should not ever need to use this. the Start() kernel usually figures this out and calls it. 
+You should not ever need to use this. the Start() kernel usually figures this out and calls it.
 
 Internal process
 1. Disconnects
@@ -531,7 +537,7 @@ Handles incoming messages ***NEED VERY GOOD DOCUMENTATION HERE***** (TODO)
 
 sub ProcessJabberMessage {
     my $self = shift;
-    DEBUG("Jabber process called");
+    DEBUG("ProcessJabberMessage called");
     my $obj_ID = $self->_get_obj_id() or return;
 
     my $session_id = shift;
@@ -557,19 +563,19 @@ sub ProcessJabberMessage {
     my $time_now = time;
     if($client_start_time{$obj_ID} > $time_now - $grace_period
        || (defined $forum_join_time{$obj_ID}{$from} && $forum_join_time{$obj_ID}{$from} > $time_now - $grace_period)) {
-	my $cond1 = "$client_start_time{$obj_ID} > $time_now - $grace_period";
-	my $cond2 = "$forum_join_time{$obj_ID}{$from} > $time_now - $grace_period";
-	DEBUG("Ignoring messages cause I'm in startup for forum $from\n"
-	      . "$cond1\n"
-	      . "$cond2");
+    my $cond1 = "$client_start_time{$obj_ID} > $time_now - $grace_period";
+    my $cond2 = "$forum_join_time{$obj_ID}{$from} > $time_now - $grace_period";
+    DEBUG("Ignoring messages cause I'm in startup for forum $from\n"
+          . "$cond1\n"
+          . "$cond2");
         return; # Ignore messages the first few seconds.
     }
 
     # Ignore Group messages with no resource on them. (Server Messages?)
     if($ignore_messages{$obj_ID}{ignore_server_messages}) {
         if($from_full !~ m/^([^\@]+)\@([^\/]+)\/(.+)$/) {
-	    DEBUG("Server message? ($from_full) - $message");
-            return if($from_full !~ m/^([^\@]+)\@([^\/]+)\//); 
+        DEBUG("Server message? ($from_full) - $message");
+            return if($from_full !~ m/^([^\@]+)\@([^\/]+)\//);
             ERROR("Couldn't recognize from_full ($from_full). Ignoring message: $body");
             return;
         }
@@ -577,11 +583,11 @@ sub ProcessJabberMessage {
 
     # Are these my own messages?
     if($ignore_messages{$obj_ID}{ignore_self_messages}) {
-	my $bot_alias = $self->get_alias();
-	if(defined $resource && $bot_alias eq $resource) { # Ignore my own messages.
-	    DEBUG("Ignoring message from self...\n");
-	    return;
-	}
+    my $bot_alias = $self->get_alias();
+    if(defined $resource && $bot_alias eq $resource) { # Ignore my own messages.
+        DEBUG("Ignoring message from self...\n");
+        return;
+    }
     }
 
     # Determine if this message was addressed to me. (groupchat only)
@@ -594,7 +600,7 @@ sub ProcessJabberMessage {
             my $qm_address_type = quotemeta($address_type);
             next if($body !~ m/^\s*$qm_address_type\s*(\S.*)$/);
             $request = $1;
-	    $bot_address_from = $address_type;
+        $bot_address_from = $address_type;
             last; # do not need to loop any more.
         }
         return if(!defined $request);
@@ -602,7 +608,7 @@ sub ProcessJabberMessage {
     }
 
     # Call the message callback if it's defined.
-    if( exists $message_function{$obj_ID} ) { 
+    if( exists $message_function{$obj_ID} ) {
         $message_function{$obj_ID}->(bot_object => $self,
                                      from_full => $from_full,
                                      body => $body,
@@ -646,8 +652,8 @@ sub get_responses {
     my $forum = shift;
 
     if(!defined $forum) {
-	WARN("No forum supplied for get_responses()");
-	return;
+    WARN("No forum supplied for get_responses()");
+    return;
     }
 
     my @aliases_to_respond_to;
@@ -666,10 +672,10 @@ sub Version : PRIVATE {
 
     my $iq = new Net::XMPP::IQ();
     $iq->SetIQ(to=> 'todd.e.rinaldo@mx-dev.jpmorgan.com/Shiva'
-	       , from=> 'murex.bot@mx-dev.jpmorgan.com/Murex-Bot'
-	       , id=>   'jcl_122'
-	       , type=> 'get'
-	      );
+           , from=> 'murex.bot@mx-dev.jpmorgan.com/Murex-Bot'
+           , id=>   'jcl_122'
+           , type=> 'get'
+          );
     my $iqType = $iq->NewChild( 'jabber:iq:version' );
     DEBUG("Sending IQ Message:" . $iq->GetXML());
     $jabber_client{$obj_ID}->Send($iq)
@@ -677,7 +683,7 @@ sub Version : PRIVATE {
 
 =item B<InIQ> - DO NOT CALL
 
-Called when the client receives new messages during Process of this type. 
+Called when the client receives new messages during Process of this type.
 
 =cut
 
@@ -696,14 +702,14 @@ sub InIQ {
     my $iqReply;
 
     if($xmlns eq "jabber:iq:version") {
-	return;
-	$iqReply = new Net::XMPP::IQ();
-	my $iqType = $iqReply->NewChild( 'jabber:iq:version' );
-	$iqType->Setname("test");
-	#    $iqReply->Set("name", "Perl");
-	DEBUG("version");
+    return;
+    $iqReply = new Net::XMPP::IQ();
+    my $iqType = $iqReply->NewChild( 'jabber:iq:version' );
+    $iqType->Setname("test");
+    #    $iqReply->Set("name", "Perl");
+    DEBUG("version");
     } else {
-	return;
+    return;
     }
 
     DEBUG("Reply: ", $iqReply->GetXML());
@@ -719,10 +725,10 @@ sub InIQ {
 
 =item B<JabberPresenceMessage> - DO NOT CALL
 
-Called when the client receives new presence messages during Process. 
+Called when the client receives new presence messages during Process.
 Mostly we are just pushing the data down into the client DB for later processing.
 
-=cut 
+=cut
 
 sub JabberPresenceMessage {
     my $self = shift;
@@ -735,7 +741,7 @@ sub JabberPresenceMessage {
     if($type eq 'subscribe') { # Always allow people to subscribe to us. Why wouldn't we?
         my $from = $presence->GetFrom();
         $jabber_client{$obj_ID}->Subscription(type=>"subscribed",
-                          				      to=>$from);
+                                              to=>$from);
         INFO("Processed subscription request from $from");
         return;
     } elsif($type eq 'unsubscribe') { # Always allow people to subscribe to us. Why wouldn't we?
@@ -810,11 +816,11 @@ sub get_safety_mode {
 
     # Must be in safety mode and all thresholds met.
     my $mode = !!($safety_mode{$obj_ID}
-		  && $message_delay{$obj_ID} >= 1/5
-		  && $max_message_size{$obj_ID} <= 1000
-		  && $max_message_size{$obj_ID} <= 166
-		  && $ignore_messages{$obj_ID}{ignore_self_messages}
-		 );
+          && $message_delay{$obj_ID} >= 1/5
+          && $max_message_size{$obj_ID} <= 1000
+          && $max_message_size{$obj_ID} <= 166
+          && $ignore_messages{$obj_ID}{ignore_self_messages}
+         );
     return $mode;
 }
 
@@ -848,8 +854,8 @@ $recipient must read as user@server/Resource or it will not send.
 =cut
 
     sub SendPersonalMessage {
-	my $self = shift;
-	my $recipient = shift;
+    my $self = shift;
+    my $recipient = shift;
     my $message = shift;
 
     return $self->SendJabberMessage($recipient, $message, 'chat');
@@ -878,16 +884,16 @@ sub SendJabberMessage {
 
     # Split the message into no more than max_message_size so that we do not piss off jabber.
     # Split on new line. Space if you have to or just chop at max size.
-    my @message_chunks = ( $message =~ /.{1,$max_size}$|.{1,$max_size}\n|.{1,$max_size}\s|.{1,$max_size}/gs ); 
+    my @message_chunks = ( $message =~ /.{1,$max_size}$|.{1,$max_size}\n|.{1,$max_size}\s|.{1,$max_size}/gs );
 
 
     DEBUG("Max message = $max_size. Splitting...") if($#message_chunks > 0);
     my $return_value;
     foreach my $message_chunk (@message_chunks) {
-	my $msg_return = $self->_SendIndividualMessage($recipient, $message_chunk, $message_type, $subject);
-	if(defined $msg_return) {
-	    $return_value .= $msg_return;
-	}
+        my $msg_return = $self->_SendIndividualMessage($recipient, $message_chunk, $message_type, $subject);
+        if(defined $msg_return) {
+            $return_value .= $msg_return;
+        }
     }
     return $return_value;
 }
@@ -902,7 +908,7 @@ sub SendJabberMessage {
 sub _SendIndividualMessage : PRIVATE {
     my $self = shift;
     my $obj_ID = $self->_get_obj_id() or return "Not an object\n"; #Failure
-    
+
     my $recipient = shift;
     my $message_chunk = shift;
     my $message_type = shift;
@@ -917,11 +923,11 @@ sub _SendIndividualMessage : PRIVATE {
     ERROR('$recipient not defined!');
     return "No recipient!\n";
     }
-    
+
     my $yday = (localtime)[7];
     my $hour = (localtime)[2];
     my $messages_this_hour = ++$messages_sent_today{$obj_ID}{$yday}{$hour};
-    
+
     if($messages_this_hour > $max_messages_per_hour{$obj_ID}) {
     $subject = "" if(!defined $subject); # Keep warning messages quiet.
     $message_chunk = "" if(!defined $message_chunk); # Keep warning messages quiet.
@@ -933,7 +939,7 @@ sub _SendIndividualMessage : PRIVATE {
           . "Message sent:\n"
           . "$message_chunk"
           );
-    
+
     # Send 1 panic message out to jabber if this is our last message before quieting down.
     return "Too many messages ($messages_this_hour)\n";
     }
@@ -952,8 +958,8 @@ sub _SendIndividualMessage : PRIVATE {
     DEBUG("Sleeping $message_delay{$obj_ID} after sending message.");
     Time::HiRes::sleep $message_delay{$obj_ID}; #Throttle messages.
 
-    if($messages_this_hour == $max_messages_per_hour{$obj_ID}) { 
-    $jabber_client{$obj_ID}->MessageSend(to => $recipient
+    if($messages_this_hour == $max_messages_per_hour{$obj_ID}) {
+        $jabber_client{$obj_ID}->MessageSend(to => $recipient
                          , body => "Cannot send more messages this hour. "
                          . "$messages_this_hour of $max_messages_per_hour{$obj_ID} already sent."
                          , type => $message_type
@@ -978,11 +984,11 @@ sub SetForumSubject {
     my $subject = shift;
 
     if(length $subject > $max_message_size{$obj_ID}) {
-	my $subject_len = length($subject);
-	ERROR("Someone tried to send a subject message $subject_len bytes long!");
-	my $subject = substr($subject, 0, $max_message_size{$obj_ID});
-	DEBUG("Truncated subject: $subject");
-	return "Subject is too long!";
+    my $subject_len = length($subject);
+    ERROR("Someone tried to send a subject message $subject_len bytes long!");
+    my $subject = substr($subject, 0, $max_message_size{$obj_ID});
+    DEBUG("Truncated subject: $subject");
+    return "Subject is too long!";
     }
     $self->_SendIndividualMessage($recipient, "Setting subject to $subject", 'groupchat', $subject);
 
